@@ -16,11 +16,17 @@ from muteria.drivers.testgeneration.testcases_info import TestcasesInfoObject
 from muteria.drivers import DriversUtils
 from muteria.drivers.testgeneration.testcase_formats.ktest.ktest \
                                                         import KTestTestFormat
+from muteria.drivers.testgeneration.testcase_formats.ktest.utils \
+                                         import ConvertCollectKtestsSeeds, Misc
+from muteria.drivers.testgeneration.tools_by_languages.c.klee.driver_config \
+                                                        import DriverConfigKlee
 
 ERROR_HANDLER = common_mix.ErrorHandler
 
 class TestcasesToolKlee(BaseTestcaseTool):
 
+    SEED_DIR_ARG_NAME = 'seed-out-dir' #Newer klee is #'seed-dir'
+
     @classmethod
     def installed(cls, custom_binary_dir=None):
         """ Check that the tool is installed
@@ -41,6 +47,18 @@ class TestcasesToolKlee(BaseTestcaseTool):
 
     def __init__(self, *args, **kwargs):
         BaseTestcaseTool.__init__(self, *args, **kwargs)
+
+        self.driver_config = None
+        if self.config.get_tool_user_custom() is not None:
+            self.driver_config = \
+                            self.config.get_tool_user_custom().DRIVER_CONFIG
+        if self.driver_config is None:
+            self.driver_config = DriverConfigKlee()
+        else:
+            ERROR_HANDLER.assert_true(isinstance(self.driver_config, \
+                                        DriverConfigKlee),\
+                                            "invalid driver config", __file__)
+
         self.test_details_file = \
                     os.path.join(self.tests_working_dir, 'test_details.json')
         self.klee_used_tmp_build_dir = os.path.join(self.tests_working_dir, \
@@ -51,6 +69,18 @@ class TestcasesToolKlee(BaseTestcaseTool):
 
         self.keptktest2dupktests = os.path.join(self.tests_working_dir, \
                                                 'kept_to_dup_ktests_map.json')
+        
+        self.ktest_with_must_exist_dir_file = os.path.join(\
+                                                self.tests_storage_dir, \
+                                                'ktest_to_must_exist_dir.json')
+        if os.path.isfile(self.ktest_with_must_exist_dir_file):
+            self.ktest_with_must_exist_dir = common_fs.loadJSON(\
+                                          self.ktest_with_must_exist_dir_file)
+        else:
+            self.ktest_with_must_exist_dir = {}
+
+        self.gen_tests_no_dup_with_seeds = \
+                        self.driver_config.get_gen_tests_no_dup_with_seeds ()
 
         if os.path.isdir(self.klee_used_tmp_build_dir):
             shutil.rmtree(self.klee_used_tmp_build_dir)
@@ -77,23 +107,60 @@ class TestcasesToolKlee(BaseTestcaseTool):
             '-libc': 'uclibc',
             '-max-sym-array-size': '4096',
             '-max-instruction-time': '10.',
+            '-'+self.SEED_DIR_ARG_NAME: None, 
         }
         return bool_params, key_val_params
     #~ def _get_default_params()
     
+    @staticmethod
+    def _validate_passed_sym_args(raw_sym_args):
+        """ check that the sym args are properly formated
+        """
+        pb = []
+        for g in raw_sym_args:
+            if type(g) not in (list, tuple):
+                pb.append(g)
+            else:
+                for elem in g:
+                    if type(elem) != str or ' ' in elem:
+                        pb.append(g)
+        ERROR_HANDLER.assert_true(len(pb) == 0, \
+                "There are missformed passed sym args: {}".format(pb), \
+                                                                __file__)
+    #~ def _validate_passed_sym_args()
+    
     # SHADOW override
-    def _get_sym_args(self):
+    def _get_sym_args(self, cfg_args):
         # sym args
         default_sym_args = ['-sym-arg', '5']
 
-        klee_sym_args = default_sym_args
+        klee_sym_args = None
         uc = self.config.get_tool_user_custom()
         if uc is not None:
             post_bc_cmd = uc.POST_TARGET_CMD_ORDERED_FLAGS_LIST
             if post_bc_cmd is not None:
+                self._validate_passed_sym_args(post_bc_cmd)
                 klee_sym_args = []
                 for tup in post_bc_cmd:
                     klee_sym_args += list(tup)
+
+        # Use seeds's (merge with the specified insdead of override)            
+        seed_dir = self.get_value_in_arglist(cfg_args, self.SEED_DIR_ARG_NAME)
+        if seed_dir is not None:
+            cv = ConvertCollectKtestsSeeds(\
+                                    custom_binary_dir=self.custom_binary_dir)
+            grouped_klee_sym_args = cv.get_ktests_sym_args(seed_dir, \
+                                        compressed=seed_dir.endswith(\
+                                            ConvertCollectKtestsSeeds.tar_gz))
+            self._validate_passed_sym_args(grouped_klee_sym_args)
+            klee_sym_args = []
+            for tup in grouped_klee_sym_args:
+                klee_sym_args += list(tup)
+
+        if klee_sym_args is None:
+            klee_sym_args = default_sym_args
+
+        
         return klee_sym_args
     #~ def _get_sym_args()
 
@@ -107,6 +174,11 @@ class TestcasesToolKlee(BaseTestcaseTool):
         return None 
     #~ def _get_back_llvm_compiler_path()
 
+    # SEMu may override
+    def _get_compile_flags_list(self):
+        return None
+    #~ def _get_compile_flags_list()
+
     @staticmethod
     def get_value_in_arglist(arglist, flagname):
         value = None
@@ -153,7 +225,9 @@ class TestcasesToolKlee(BaseTestcaseTool):
         timeout_grace_period = 600
         max_time = None
         cur_max_time = float(self.get_value_in_arglist(args, 'max-time'))
-        max_time = cur_max_time + 5 #5 to give time to klee
+        #to give time to klee add FRAMEWORK GRACE
+        max_time = cur_max_time + \
+                                self.config.TEST_GEN_TIMEOUT_FRAMEWORK_GRACE
 
         # set stack to unlimited
         stack_ulimit_soft, stack_ulimit_hard = \
@@ -163,21 +237,42 @@ class TestcasesToolKlee(BaseTestcaseTool):
 
         # Execute Klee
         ret, out, err = DriversUtils.execute_and_get_retcode_out_err(\
-                                    runtool, args, timeout=max_time, \
+                                    runtool, args_list=args, timeout=max_time,\
                                     timeout_grace_period=timeout_grace_period)
                                     #out_on=False, err_on=False)
-        
+        '''o_d_dbg = self.get_value_in_arglist(args, "output-dir") #DBG
+        if os.path.isdir(o_d_dbg): #DBG
+            shutil.rmtree(o_d_dbg) #DBG
+        import subprocess #DBG
+        p = subprocess.Popen([runtool]+args, env=None, cwd=None, \
+                                                            #close_fds=True, \
+                                                        stdin=None, \
+                                                        stderr=subprocess.STDOUT, \
+                                                        stdout=None, shell=True, \
+                                                        preexec_fn=os.setsid) #DBG
+        try: #DBG
+            stdout, stderr = p.communicate(timeout=max_time) #DBG
+        except subprocess.TimeoutExpired: #DBG
+            stdout, stderr = p.communicate(timeout=max_time) #DBG
+        #os.system(" ".join([runtool]+args)) #DBG'''
+         
         # restore stack
         if stack_ulimit_soft != -1:
             resource.setrlimit(resource.RLIMIT_STACK, \
                                         (stack_ulimit_soft, stack_ulimit_hard))
 
-        if (ret != 0 and ret not in DriversUtils.EXEC_TIMED_OUT_RET_CODE):
+        if (ret != 0 and ret not in DriversUtils.EXEC_TIMED_OUT_RET_CODE \
+                and not out.rstrip().endswith(": ctrl-c detected, exiting.")):
             logging.error(out)
             logging.error(err)
             logging.error("\n>> CMD: " + " ".join([runtool]+args) + '\n')
             ERROR_HANDLER.error_exit("call to klee testgen failed, "
                                     + "error code is {}".format(ret), __file__)
+        
+        if self.driver_config.get_verbose_generation():
+            logging.debug(out)
+            logging.debug(err)
+            logging.debug("\nCMD: " + " ".join([runtool]+args))
     #~ def _call_generation_run()
 
     # SHADOW should override
@@ -193,6 +288,7 @@ class TestcasesToolKlee(BaseTestcaseTool):
                                                 meta_criteria_tool_obj=None):
         back_llvm_compiler = self._get_back_llvm_compiler() 
         back_llvm_compiler_path = self._get_back_llvm_compiler_path() 
+        compile_flags_list = self._get_compile_flags_list()
         
         pre_ret, ret, post_ret = code_builds_factory.transform_src_into_dest(\
                         src_fmt=CodeFormats.C_SOURCE,\
@@ -200,7 +296,8 @@ class TestcasesToolKlee(BaseTestcaseTool):
                         src_dest_files_paths_map=rel_path_map,\
                         compiler=back_llvm_compiler, \
                         llvm_compiler_path=back_llvm_compiler_path, \
-                        clean_tmp=True, reconfigure=True)
+                        clean_tmp=True, reconfigure=True, \
+                        flags_list=compile_flags_list)
         if ret == common_mix.GlobalConstants.COMMAND_FAILURE:
             ERROR_HANDLER.error_exit("Program {}.".format(\
                                         'LLVM built problematic'), __file__)
@@ -232,11 +329,22 @@ class TestcasesToolKlee(BaseTestcaseTool):
                 for f in files:
                     tc = os.path.normpath(os.path.join(root, f))
                     if tc.endswith(KTestTestFormat.ktest_extension):
-                        tc_info_obj.add_test(tc)
+                        gen_time = self._get_generation_time_of_test(tc, \
+                                                     self.tests_storage_dir)
+                        tc_info_obj.add_test(tc, generation_time=gen_time)
             os.chdir(cwd)
             self.testcase_info_object = tc_info_obj
             return self.testcase_info_object
     #~ def get_testcase_info_object()
+    
+    @staticmethod
+    def _get_generation_time_of_test(test, test_top_dir):
+        """ extract the generation timestamp of a test. the test and 
+            its top location are specified
+        """
+        # FIXME: implememnt this in klee
+        return None
+    #~ def _get_generation_time_of_test()
 
     def _prepare_executable(self, exe_path_map, env_vars, \
                                                         collect_output=False):
@@ -308,12 +416,32 @@ class TestcasesToolKlee(BaseTestcaseTool):
         if extra_env is not None and len(extra_env) > 0:
             env_vars = dict(env_vars).update(extra_env)
 
+        # get stdin if exists
+        stdin_file = os.path.join(self.tests_storage_dir, \
+                                        os.path.dirname(testcase), \
+                                        KTestTestFormat.STDIN_KTEST_DATA_FILE)
+        if os.path.isfile(stdin_file) and os.path.getsize(stdin_file) > 0:
+            stdin = open(stdin_file)
+        else:
+            stdin = None
+
+        if testcase in self.ktest_with_must_exist_dir and \
+                      len(self.ktest_with_must_exist_dir[testcase]) > 0:
+            must_exist_dirs = self.ktest_with_must_exist_dir[testcase]
+        else:
+            must_exist_dirs = None
+        
         verdict = KTestTestFormat.execute_test(local_exe, \
                         os.path.join(self.tests_storage_dir, testcase), \
                         env_vars=env_vars, \
+                        stdin=stdin, \
+                        must_exist_dir_list=must_exist_dirs, \
                         timeout=timeout, \
                         collected_output=collected_output, \
                         custom_replay_tool_binary_dir=self.custom_binary_dir)
+        
+        if stdin is not None:
+            stdin.close()
 
         return verdict, collected_output
     #~ def _execute_a_test()
@@ -401,27 +529,39 @@ class TestcasesToolKlee(BaseTestcaseTool):
 
         args.append(bitcode_file)
 
-        args += self._get_sym_args()
+        args += self._get_sym_args(args)
 
         self._call_generation_run(prog, list(args))
 
         # XXX: remove duplicate tests
         kepttest2duptest_map = {}
-        dup_list, invalid = KTestTestFormat.ktest_fdupes(\
-                                                    self.tests_storage_dir, \
+        folders = [self.tests_storage_dir]
+        if self.gen_tests_no_dup_with_seeds:
+            seed_dir = self.get_value_in_arglist(args, self.SEED_DIR_ARG_NAME)
+            if seed_dir is not None:
+                seed_dir = os.path.normpath(os.path.abspath(seed_dir))
+                folders.append(seed_dir)
+
+        dup_list, invalid = KTestTestFormat.ktest_fdupes(*folders, \
                         custom_replay_tool_binary_dir=self.custom_binary_dir)
         if len(invalid) > 0:
             logging.warning(\
-                        "{} generated ktests are invalid".format(len(invalid)))
+                        "{} generated ktests are invalid ({})".format(\
+                                                        len(invalid), invalid))
             for kt in invalid:
-                os.remove(kt)
+                if KTestTestFormat.get_dir(kt, folders) == \
+                                                        self.tests_storage_dir:
+                    os.remove(kt)
         for dup_tuple in dup_list:
-            kepttest2duptest_map[os.path.relpath(\
-                            dup_tuple[0], \
-                            self.tests_storage_dir)] = [os.path.relpath(dp) \
-                                                    for dp in dup_tuple[1:]]
+            key = os.path.relpath(dup_tuple[0], \
+                                KTestTestFormat.get_dir(dup_tuple[0], folders))
+            kepttest2duptest_map[key] = [os.path.relpath(dp, \
+                                        KTestTestFormat.get_dir(dp, folders)) \
+                                                for dp in dup_tuple[1:]]
             for df in dup_tuple[1:]:
-                os.remove(df)
+                if KTestTestFormat.get_dir(df, folders) == \
+                                                        self.tests_storage_dir:
+                    os.remove(df)
         common_fs.dumpJSON(kepttest2duptest_map, self.keptktest2dupktests)
         
         # Copy replay tool into test folder
@@ -431,9 +571,29 @@ class TestcasesToolKlee(BaseTestcaseTool):
 
         store_obj = {repo_rel_exe_file: os.path.basename(bitcode_file)}
         common_fs.dumpJSON(store_obj, self.test_details_file)
+        
+        # Compute must exist dirs
+        test2file = {}
+        for kt in self.get_testcase_info_object().get_tests_list():
+            test2file[kt] = os.path.join(self.tests_storage_dir, kt)
+        ktest2reqdir = Misc.get_must_exist_dirs_of_ktests(test2file, \
+                                                    self.custom_binary_dir)
+        ktest2reqdir = {kt: v for kt, v in ktest2reqdir.items() if len(v) > 0}
+        self.ktest_with_must_exist_dir = ktest2reqdir
+        if len(ktest2reqdir) > 0:
+            common_fs.dumpJSON(ktest2reqdir, \
+                                        self.ktest_with_must_exist_dir_file)
     #~ def _do_generate_tests()
 
+    def get_ktests_dir(self):
+        return self.tests_storage_dir
+    #~ def get_ktests_dir()
+
     def can_run_tests_in_parallel(self):
         return True
     #~ def can_run_tests_in_parallel()
-#~ class TestcasesToolKlee
\ No newline at end of file
+
+    def get_test_format_class (self):
+        return KTestTestFormat
+    # def get_test_format_class ()
+#~ class TestcasesToolKlee
